#!/usr/bin/env python
# -*- coding: utf-8 -*-

#######################################################################
# maintainer: <schomi@vuplus-support.org>
# This plugin is free software, you are allowed to
# modify it (if you keep the license),
# but you are not allowed to distribute/publish
# it without source code (this version and your modifications).
# This means you also have to distribute
# source code of your modifications.
#######################################################################
'''The code written by Schomi
extended by mfaraj57
-settings added to select target device,/media/usb is available in original plugin
-more info about free and total space of flash and target device
-update size info after moving any plugin
-removed bytes2human modules and replaced by local functions
'''
# recode lululla
from . import _
from .plugin import cfg_mnt, PluginMoverScreen, storagedevicescreen

from Components.ActionMap import ActionMap
from Components.Label import Label
from Components.Pixmap import Pixmap
from Components.Sources.List import List
from Components.Sources.StaticText import StaticText
from Screens.MessageBox import MessageBox
from Screens.Screen import Screen
from Tools.Directories import SCOPE_PLUGINS, resolveFilename
from Tools.LoadPixmap import LoadPixmap
from enigma import eTimer
from os import listdir, remove, path, symlink, chdir, statvfs
from os import path as os_path
from os import walk as os_walk
from os.path import join, getsize
import shutil

pname = _("PluginSkinMover")
pdesc = _("Move plugins and skins between flash memory and pen drive")
pversion = "1.0"  # extended by mfaraj57
pdate = "20241103"
# recode lululla


def foldersize(size):
    try:
        fspace = round(float((size) // (1024.0 * 1024.0)), 2)
        # tspace=round(float((capacity) / (1024.0 * 1024.0)), 1)
        spacestr = str(fspace) + 'MB'
        return spacestr
    except:
        return ""


def freespace(folder='/'):
    try:
        diskSpace = statvfs(folder)
        capacity = float(diskSpace.f_bsize * diskSpace.f_blocks)
        available = float(diskSpace.f_bsize * diskSpace.f_bavail)
        fspace = round(float((available) // (1024.0 * 1024.0)), 2)
        tspace = round(float((capacity) // (1024.0 * 1024.0)), 1)
        spacestr = 'Free space(' + str(fspace) + 'MB) Total space(' + str(tspace) + 'MB)'
        return spacestr
    except:
        return "location is unavaiable or not mounted"


class SkinMoverScreen(Screen):
    skin = """
        <screen position="center,center" size="1000,520" title="SkinMoverScreen" flags="wfNoBorder">
            <widget name="info" position="10,5" size="830,70" font="Regular;24" foregroundColor="#00fff000" />
            <widget source="menu" render="Listbox" position="10,100" size="630,360" zPosition="5" scrollbarMode="showOnDemand" transparent="1">
                <convert type="TemplatedMultiContent">
                    {"template":
                        [
                            MultiContentEntryPixmapAlphaTest(pos = (2, 2), size = (54, 54), png = 1),
                            MultiContentEntryText(pos = (60, 13), size = (100, 30), font=0, flags = RT_HALIGN_LEFT|RT_VALIGN_CENTER, text = 2),
                            MultiContentEntryText(pos = (170, 13), size = (450, 30), font=0, flags = RT_HALIGN_LEFT|RT_VALIGN_CENTER, text = 0),
                        ],
                        "fonts": [gFont("Regular", 24),gFont("Regular", 16)],
                        "itemHeight": 60
                    }
                </convert>
            </widget>
            <widget name="Picture" position="691,149" size="280,210" alphatest="on" />
            <ePixmap position="10,460" size="25,25" zPosition="0" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/PluginSkinMover/pic/button_red.png" transparent="1" alphatest="on" />
            <ePixmap position="240,460" size="25,25" zPosition="0" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/PluginSkinMover/pic/button_green.png" transparent="1" alphatest="on" />
            <ePixmap position="495,460" size="25,25" zPosition="0" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/PluginSkinMover/pic/button_yellow.png" transparent="1" alphatest="on" />
            <ePixmap position="750,460" size="25,25" zPosition="0" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/PluginSkinMover/pic/button_blue.png" transparent="1" alphatest="on" />
            <widget source="key_red" render="Label" position="40,455" size="185,40" zPosition="1" font="Regular;24" halign="left" transparent="1" />
            <widget source="key_green" render="Label" position="275,455" size="185,40" zPosition="1" font="Regular;24" halign="left" transparent="1" />
            <widget source="key_yellow" render="Label" position="530,455" size="185,40" zPosition="1" font="Regular;24" halign="left" transparent="1" />
            <widget source="key_blue" render="Label" position="785,455" size="185,40" zPosition="1" font="Regular;24" halign="left" transparent="1" />
        </screen>"""

    def __init__(self, session):
        Screen.__init__(self, session)
        self.skin_path = resolveFilename(SCOPE_PLUGINS, "Extensions/PluginSkinMover")
        self.session = session
        self.title = pname + " (" + pversion + ")"
        try:
            self["title"] = StaticText(self.title)
        except:
            print('self["title"] was not found in skin')
        self["info"] = Label("Please wait..")
        self["Picture"] = Pixmap()
        self["key_red"] = StaticText(_("Exit"))
        self["key_green"] = StaticText(_("in Flash"))
        self["key_yellow"] = StaticText(_("Settings"))
        self["key_blue"] = StaticText(_("PluginMover"))
        menu_list = []
        self["menu"] = List(menu_list)
        self["shortcuts"] = ActionMap(["OkCancelActions", "ColorActions", "DirectionActions"],
                                      {"ok": self.startmoving,
                                       "cancel": self.keyCancel,
                                       "red": self.keyCancel,
                                       "green": self.startmoving,
                                       "blue": self.pluginmover,
                                       "yellow": self.showsettings}, -2)

        self.plugin_base_dir = "/usr/share/enigma2"
        try:
            targetlocation = str(cfg_mnt)  # config.PluginSkinMover.targetdevice.value
        except:
            targetlocation = "/media/usb"
        self.mount_dir = targetlocation
        self.ext_dir = targetlocation + "/enigma2"
        self.enabled_pic = LoadPixmap(cached=True, path=resolveFilename(SCOPE_PLUGINS, "Extensions/PluginSkinMover/pic/loc_flash.png"))
        self.disabled_pic = LoadPixmap(cached=True, path=resolveFilename(SCOPE_PLUGINS, "Extensions/PluginSkinMover/pic/loc_media.png"))
        if self.selectionChanged not in self["menu"].onSelectionChanged:
            self["menu"].onSelectionChanged.append(self.selectionChanged)
        self.timer = eTimer()
        try:
            self.timer_conn = self.timer.timeout.connect(self.createMenuList)
        except:
            self.timer.callback.append(self.createMenuList)
        self.timer.start(100, True)

    def startmoving(self):
        self["info"].setText("Moving skin,please wait...")
        self.timer = eTimer()
        try:
            self.timer_conn = self.timer.timeout.connect(self.runMenuEntry)
        except:
            self.timer.callback.append(self.runMenuEntry)
        self.timer.start(100, True)

    def pluginmover(self):
        # from .plugin import PluginMoverScreen
        self.session.open(PluginMoverScreen)

    def showsettings(self):
        # from .settings import storagedevicescreen
        self.session.openWithCallback(self.selectionChanged, storagedevicescreen)

    def selectionChanged(self, result=None):
        if result is True:
            self.createMenuList()
        try:
            sel = self["menu"].getCurrent()
        except:
            return
        try:
            self.setPicture(sel[0])
            if sel[1] == self.enabled_pic:
                self["key_green"].setText(_("in Flash"))
            elif sel[1] == self.disabled_pic:
                self["key_green"].setText(_("Exported"))
        except:
            pass
        self.getdevices_sizes()

    # Flash size
    def getdevices_sizes(self):
        freeflash = 'Unable to read flash size'
        try:
            # stat = statvfs("/")
            freeflash = freespace("/")  # bytes2human(freeflash, 1)
        except:
            pass
        # Device size
        freedev = "The device is unavailable or not mounted"
        try:
            # stat = statvfs(self.mount_dir)
            devicelocation = str(cfg_mnt)  # config.PluginSkinMover.targetdevice.value
            self.mount_dir = devicelocation
            freedev = devicelocation + " " + freespace(devicelocation)
        except:
            try:
                devicelocation = str(cfg_mnt)  # config.PluginSkinMover.targetdevice.value
            except:
                devicelocation = 'unknown'
            freedev = devicelocation + " " + freedev
            pass
        self["info"].setText(_("Flash: %s \nDevice: %s") % (freeflash, freedev))

    def createMenuList(self):
        self["info"].setText(_("read ..."))
        chdir(self.plugin_base_dir)
        f_list = []
        list_dir = sorted(listdir(self.plugin_base_dir))
        # print list_dir
        for f in list_dir:
            linked_dir = self.plugin_base_dir + "/" + f
            if os_path.isdir(linked_dir):
                if not os_path.exists(linked_dir + '/skin.xml'):
                    continue

                # Flash size
                # try:
                    # stat = statvfs(linked_dir)
                # except OSError:
                    # return -1
                free = self.GetFolderSize(linked_dir)
                size = str(foldersize(free))
                # pic
                pic = self.enabled_pic

                if os_path.exists(linked_dir):
                    if os_path.islink(linked_dir):
                        pic = self.disabled_pic
                    else:
                        pic = self.enabled_pic
                    f_list.append((f, pic, size))
        menu_list = []
        for entry in f_list:
            menu_list.append((entry[0], entry[1], str(entry[2])))
        self["menu"].updateList(menu_list)
        self.selectionChanged(False)

    def setPicture(self, f):
        preview = self.plugin_base_dir + "/" + f + "/" + "prev.png"
        if not path.exists(preview):
            preview = self.plugin_base_dir + "/" + f + "/" + f.lower() + ".png"
        if path.exists(preview):
            self["Picture"].instance.setPixmapFromFile(preview)
            self["Picture"].show()
        else:
            self["Picture"].hide()

    def keyCancel(self):
        self.close()

    def runMenuEntry(self):
        try:
            devicelocation = str(cfg_mnt)  # config.PluginSkinMover.targetdevice.value
            self.mount_dir = devicelocation
            print("242", self.mount_dir)
        except:
            pass
        if os_path.ismount(self.mount_dir):
            self["info"].setText(_("Moving, please wait ..."))
            sel = self["menu"].getCurrent()
            inflash = self.plugin_base_dir + "/" + sel[0]
            self.ext_dir = self.mount_dir + "/enigma2"
            notinflash = self.ext_dir + "/" + sel[0]
            # print "[PluginSkinMover] " + inflash
            # print "[PluginSkinMover] " + notinflash
            # print "[PluginSkinMover] Start movement!"
            error = False
            if sel[1] == self.enabled_pic:
                if os_path.exists(notinflash):
                    shutil.rmtree(notinflash)
                # debug = True
                try:
                    shutil.copytree(inflash, notinflash)
                    error = False
                except:
                    error = True
                    # print "[PluginSkinMover] Error during movement!"
                if error is False:
                    try:
                        shutil.rmtree(inflash)
                        symlink(notinflash, inflash)
                    except:
                        pass
            elif sel[1] == self.disabled_pic:
                if os_path.islink(inflash):
                    remove(inflash)
                    # debug = True
                    try:
                        shutil.copytree(notinflash, inflash)
                        error = False
                    except:
                        error = True
                        # print "[PluginSkinMover] Error during movement!"
                    if error is False:
                        try:
                            shutil.rmtree(notinflash)
                        except:
                            pass
            if error:
                self.session.open(MessageBox, _("Skin movement was not successful, please check devices!"), type=MessageBox.TYPE_ERROR, timeout=10)
                error = False

            self.createMenuList()
        else:
            self.session.open(MessageBox, _("No device to %s mounted. Skin movement is not possible!") % self.mount_dir, type=MessageBox.TYPE_ERROR, timeout=10)
        self.selectionChanged()

    def GetFolderSize(self, path):
        TotalSize = 0
        for item in os_walk(path):
            for file in item[2]:
                try:
                    TotalSize = TotalSize + getsize(join(item[0], file))
                except:
                    print("error with file:  " + join(item[0], file))
        return TotalSize
